pub type MatrixReport = Vec<Row>;
pub type SingleRow = std::iter::Once<Row>;

use prettytable::Row;

use super::{row_builder::RowBuilder, BenchmarkResult};
use std::time::Duration;

pub fn create_header() -> SingleRow {
    std::iter::once(RowBuilder::header().into_row())
}

pub fn create_sum_row(sum: Duration) -> SingleRow {
    std::iter::once(RowBuilder::total(sum).into_row())
}

pub fn create_average_row(average: Duration) -> SingleRow {
    std::iter::once(RowBuilder::average(average).into_row())
}

pub fn create_rows_for_every_solutions(
    loaded: impl IntoIterator<Item = BenchmarkResult>,
    mut on_ok: impl FnMut(Duration),
) -> MatrixReport {
    loaded
        .into_iter()
        .map(|result| {
            result
                .inspect(|on_success| on_ok(on_success.how_long()))
                .map(RowBuilder::from)
                .unwrap_or_else(RowBuilder::from)
        })
        .map(RowBuilder::into_row)
        .collect()
}
