use std::fmt::Display;

use prettytable::Table;

pub mod benchmarking;
pub mod cli;
pub mod constants;
pub mod solutions;
pub mod solving_given;

pub type AppError = anyhow::Error;
pub type AppResult<T = ()> = anyhow::Result<T>;

#[derive(Debug)]
pub enum AppOutput {
    JustString(String),
    PrettyTable(Table),
}

impl AppOutput {
    pub fn print_std_out(&self) {
        match self {
            AppOutput::JustString(string) => println!("{}", string),
            AppOutput::PrettyTable(table) => table.printstd(),
        }
    }
}

impl Display for AppOutput {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            AppOutput::JustString(string) => f.write_str(string),
            AppOutput::PrettyTable(table) => f.write_str(&table.to_string()),
        }
    }
}
