use std::process::ExitCode;

use advent_of_code_2023::{
    benchmarking::execute_benchmark,
    cli::{AppCliArgs, AppCliSubcommands},
    solving_given::solve_given_from_cli,
    AppOutput, AppResult,
};
use clap::Parser;

fn main() -> ExitCode {
    let args = AppCliArgs::parse();
    env_logger::init();
    let solution = handle_command(&args);
    match solution {
        Ok(found_solution) => {
            found_solution.print_std_out();
            ExitCode::SUCCESS
        }
        Err(error) => {
            eprintln!("{}", error);
            ExitCode::FAILURE
        }
    }
}

fn handle_command(args: &AppCliArgs) -> AppResult<AppOutput> {
    match args.sub_command() {
        AppCliSubcommands::Solve(to_solve) => {
            solve_given_from_cli(to_solve).map(AppOutput::JustString)
        }
        AppCliSubcommands::Benchmark(benchmark) => {
            execute_benchmark(benchmark).map(AppOutput::PrettyTable)
        }
    }
}
