use derive_more::derive::Into;

#[derive(Debug, PartialEq, Eq, Into, Clone, Copy)]
pub struct NonZeroCount(usize);

impl std::ops::Sub for NonZeroCount {
    type Output = Option<Self>;

    fn sub(self, rhs: Self) -> Self::Output {
        assert!(self.0 >= rhs.0);

        Self::new(self.0 - rhs.0)
    }
}

impl NonZeroCount {
    pub fn one() -> Self {
        Self(1)
    }

    pub fn decrement(&self) -> Option<Self> {
        *self - Self::one()
    }

    pub fn new(value: usize) -> Option<Self> {
        if value > 0 {
            Some(Self(value))
        } else {
            None
        }
    }
}
